searchNodes=[{"type":"module","title":"Multiverses","doc":"Elixir introduces into the world of programming, the &quot;multiverse testing&quot; pattern. This is a pattern where integration tests are run concurrently and each test sees a shard of global state. Pre-Existing Examples: Mox : each test has access to the global module mock, sharded by the pid of the running test. Ecto : each test has access to a &quot;database sandbox&quot;, which is a checked out transaction on the global database that acts as its own database shard. Hound , Wallaby : each test generates an ID that is passed outside of the BEAM that is reintercepted on ingress, this ID is then used to connect ecto sandboxes to the parent test PID This library implements Multiverses-aware versions of several constructs in the Elixir Standard Library that aren't natively Multiversable. For plugins that are provided for other systems, see the libraries: :multiverses_http - which extends this to HTTP requests that exit the BEAM. :multiverses_pubsub - which extends this to Phoenix.PubSub The following multiverse modules are provided by this core package: Multiverses.Application - which shards the application environment variables Multiverses.Registry - which shards Registries Usage In mix.exs , add the following dependency: { :multiverses , &quot;~&gt; 0.11.0&quot; , only : :test } In your code For example, if you would like to use the Multiverses version of the Application module ( Multiverses.Application ), add the following lines: To config/config.exs : config :my_app , Application , Application To config/test.exs : config :my_app , Application , Multiverses.Application To the module where you would like to use multiverses Application : @application Application . compile_env ( :my_app , Application ) And where you would like to make a multiverses Application call: def some_function do value = @application . get_env ( :my_app , :some_env_variable ) # ... end In your tests Register the module you'd like to substitute with multiverses. setup do Multiverses . shard ( Application ) end Your tests have segregated application values! defmodule MyModule do @application Multiverses.Application def get_and_wait ( value ) do @application . put_env ( :my_app , :env , value ) Process . sleep ( 1000 ) @application . get_env ( :my_app , :env ) end end defmodule SomeTest do use ExUnit.Case , async : true test do assert :foo == MyModule . get_and_wait ( :foo ) end end defmodule SomeOtherTest do use ExUnit.Case , async : true test do assert :bar == MyModule . get_and_wait ( :bar ) end end","ref":"Multiverses.html"},{"type":"function","title":"Multiverses.all/1","doc":"See Multiverses.Server.all/1 .","ref":"Multiverses.html#all/1"},{"type":"function","title":"Multiverses.allow/2","doc":"Utility version of allow/3 that lets you batch-assign multiple allowances","ref":"Multiverses.html#allow/2"},{"type":"function","title":"Multiverses.allow/3","doc":"Inspired by Mox.allow/3 , this function assigns a process or registered name process to be put into the shard of a pid or directly into a shard. usage The following is the most common use case, called from the test process, where you want to allow a spawned process to be put into the same shard as the test process. Multiverses . allow ( Application , self ( ) , child_pid ) In some cases you may want an allowance to occur from within the process that needs access to the shard. In this case, do the following: Multiverses . allow ( Application , multiverse_id , self ( ) )","ref":"Multiverses.html#allow/3"},{"type":"function","title":"Multiverses.allow_for/3","doc":"Temporarily assigns the running process to the shard, within the scope of the provided lambda. This is done through the process dictionary. Other processes will not be aware that this process has been added to the shard.","ref":"Multiverses.html#allow_for/3"},{"type":"function","title":"Multiverses.id/2","doc":"Obtains the universe id for the current process. This is found by checking process and the entries in the :$callers process dictionary entry to find if any of them are registered. If the current process is not registered, then it raises Multiverses.UnexpectedCallError Options :strict (defaults to true ): if false , returns nil , instead of crashing.","ref":"Multiverses.html#id/2"},{"type":"function","title":"Multiverses.shard/1","doc":"Creates a new shard for a particular domain module and assigns this pid to the shard. You can batch assigning multiple shards as well.","ref":"Multiverses.html#shard/1"},{"type":"function","title":"Multiverses.shards/1","doc":"Returns a list of multiverse domain modules and the respective shard-ids associated with those domain modules.","ref":"Multiverses.html#shards/1"},{"type":"type","title":"Multiverses.id/0","doc":"","ref":"Multiverses.html#t:id/0"},{"type":"module","title":"Multiverses.AppSupervisor","doc":"AppSupervisor is the core Application module that supervises the Multiverses.Server module, which should be running in active Multiverses environments.","ref":"Multiverses.AppSupervisor.html"},{"type":"function","title":"Multiverses.AppSupervisor.start/2","doc":"Callback implementation for Application.start/2 .","ref":"Multiverses.AppSupervisor.html#start/2"},{"type":"module","title":"Multiverses.Application","doc":"This module is intended to be a drop-in replacement for Application . When you drop this module in, functions relating to runtime environment variables have been substituted with equivalent functions that respect the Multiverse pattern. Warning This module is as dangerous as it is useful, so please make sure you know what you're doing before you use this . Always test your system in a staging system that has :use_multiverses unset, before deploying code that uses this module. The functions calls which take options ( :timeout , :persist ) are not supported, since it's likely that if you're using these options, you're probably not in a situation where you need multiverses. For the same reason, :get_all_env and :put_all_env are not supported and will default the Elixir standard. How it works","ref":"Multiverses.Application.html"},{"type":"function","title":"Multiverses.Application.app_dir/1","doc":"See Application.app_dir/1 .","ref":"Multiverses.Application.html#app_dir/1"},{"type":"function","title":"Multiverses.Application.app_dir/2","doc":"See Application.app_dir/2 .","ref":"Multiverses.Application.html#app_dir/2"},{"type":"function","title":"Multiverses.Application.compile_env/4","doc":"See Application.compile_env/4 .","ref":"Multiverses.Application.html#compile_env/4"},{"type":"function","title":"Multiverses.Application.compile_env!/3","doc":"See Application.compile_env!/3 .","ref":"Multiverses.Application.html#compile_env!/3"},{"type":"function","title":"Multiverses.Application.delete_env/2","doc":"See Application.delete_env/2 .","ref":"Multiverses.Application.html#delete_env/2"},{"type":"function","title":"Multiverses.Application.delete_env/3","doc":"See Application.delete_env/3 .","ref":"Multiverses.Application.html#delete_env/3"},{"type":"function","title":"Multiverses.Application.ensure_all_started/1","doc":"See Application.ensure_all_started/1 .","ref":"Multiverses.Application.html#ensure_all_started/1"},{"type":"function","title":"Multiverses.Application.ensure_all_started/2","doc":"See Application.ensure_all_started/2 .","ref":"Multiverses.Application.html#ensure_all_started/2"},{"type":"function","title":"Multiverses.Application.ensure_loaded/1","doc":"See Application.ensure_loaded/1 .","ref":"Multiverses.Application.html#ensure_loaded/1"},{"type":"function","title":"Multiverses.Application.ensure_started/1","doc":"See Application.ensure_started/1 .","ref":"Multiverses.Application.html#ensure_started/1"},{"type":"function","title":"Multiverses.Application.ensure_started/2","doc":"See Application.ensure_started/2 .","ref":"Multiverses.Application.html#ensure_started/2"},{"type":"function","title":"Multiverses.Application.fetch_env/2","doc":"See Application.fetch_env/2 .","ref":"Multiverses.Application.html#fetch_env/2"},{"type":"function","title":"Multiverses.Application.fetch_env!/2","doc":"See Application.fetch_env!/2 .","ref":"Multiverses.Application.html#fetch_env!/2"},{"type":"function","title":"Multiverses.Application.format_error/1","doc":"See Application.format_error/1 .","ref":"Multiverses.Application.html#format_error/1"},{"type":"function","title":"Multiverses.Application.get_all_env/1","doc":"See Application.get_all_env/1 .","ref":"Multiverses.Application.html#get_all_env/1"},{"type":"function","title":"Multiverses.Application.get_application/1","doc":"See Application.get_application/1 .","ref":"Multiverses.Application.html#get_application/1"},{"type":"function","title":"Multiverses.Application.get_env/2","doc":"See Application.get_env/2 .","ref":"Multiverses.Application.html#get_env/2"},{"type":"function","title":"Multiverses.Application.get_env/3","doc":"See Application.get_env/3 .","ref":"Multiverses.Application.html#get_env/3"},{"type":"function","title":"Multiverses.Application.load/1","doc":"See Application.load/1 .","ref":"Multiverses.Application.html#load/1"},{"type":"function","title":"Multiverses.Application.loaded_applications/0","doc":"See Application.loaded_applications/0 .","ref":"Multiverses.Application.html#loaded_applications/0"},{"type":"function","title":"Multiverses.Application.put_all_env/1","doc":"See Application.put_all_env/1 .","ref":"Multiverses.Application.html#put_all_env/1"},{"type":"function","title":"Multiverses.Application.put_all_env/2","doc":"See Application.put_all_env/2 .","ref":"Multiverses.Application.html#put_all_env/2"},{"type":"function","title":"Multiverses.Application.put_env/3","doc":"See Application.put_env/3 .","ref":"Multiverses.Application.html#put_env/3"},{"type":"function","title":"Multiverses.Application.put_env/4","doc":"See Application.put_env/4 .","ref":"Multiverses.Application.html#put_env/4"},{"type":"function","title":"Multiverses.Application.spec/1","doc":"See Application.spec/1 .","ref":"Multiverses.Application.html#spec/1"},{"type":"function","title":"Multiverses.Application.spec/2","doc":"See Application.spec/2 .","ref":"Multiverses.Application.html#spec/2"},{"type":"function","title":"Multiverses.Application.start/1","doc":"See Application.start/1 .","ref":"Multiverses.Application.html#start/1"},{"type":"function","title":"Multiverses.Application.start/2","doc":"See Application.start/2 .","ref":"Multiverses.Application.html#start/2"},{"type":"function","title":"Multiverses.Application.started_applications/0","doc":"See Application.started_applications/0 .","ref":"Multiverses.Application.html#started_applications/0"},{"type":"function","title":"Multiverses.Application.started_applications/1","doc":"See Application.started_applications/1 .","ref":"Multiverses.Application.html#started_applications/1"},{"type":"function","title":"Multiverses.Application.stop/1","doc":"See Application.stop/1 .","ref":"Multiverses.Application.html#stop/1"},{"type":"function","title":"Multiverses.Application.unload/1","doc":"See Application.unload/1 .","ref":"Multiverses.Application.html#unload/1"},{"type":"module","title":"Multiverses.Clone","doc":"allows a module to directly clone all of the public functions of a given module, except as delegations. thus multiverse equivalents replicate the functionality of the parent module. Consider dropping a cloned module into a test/support directory so that it can exist as compile-time for test, but not for dev or prod. Usage In the following example, FooModule has all of its functions ported into the current module as defdelegate/2 . The functions FooModule.foo/3 and FooModule.foo/4 are not, but rather should be manually implemented (if used) use Multiverses.Clone , with : FooModule , except : [ foo : 3 , foo : 4 ]","ref":"Multiverses.Clone.html"},{"type":"module","title":"Multiverses.Registry","doc":"This module is intended to be a drop-in replacement for Registry , but not all functionality is implemented. If universes are active, keys in the Registry will be {universe, key} instead of the normal key . A convenience via/2 macro has been provided, which will perform this substitution correctly. Unimplemented functionality: count_match/3,4 match/3,4 unregister_match/3,4","ref":"Multiverses.Registry.html"},{"type":"function","title":"Multiverses.Registry.child_spec/1","doc":"See Registry.child_spec/1 .","ref":"Multiverses.Registry.html#child_spec/1"},{"type":"function","title":"Multiverses.Registry.count/1","doc":"","ref":"Multiverses.Registry.html#count/1"},{"type":"function","title":"Multiverses.Registry.count_match/3","doc":"See Registry.count_match/3 .","ref":"Multiverses.Registry.html#count_match/3"},{"type":"function","title":"Multiverses.Registry.count_match/4","doc":"See Registry.count_match/4 .","ref":"Multiverses.Registry.html#count_match/4"},{"type":"function","title":"Multiverses.Registry.count_select/2","doc":"See Registry.count_select/2 .","ref":"Multiverses.Registry.html#count_select/2"},{"type":"function","title":"Multiverses.Registry.delete_meta/2","doc":"See Registry.delete_meta/2 .","ref":"Multiverses.Registry.html#delete_meta/2"},{"type":"function","title":"Multiverses.Registry.dispatch/4","doc":"","ref":"Multiverses.Registry.html#dispatch/4"},{"type":"function","title":"Multiverses.Registry.keys/2","doc":"","ref":"Multiverses.Registry.html#keys/2"},{"type":"function","title":"Multiverses.Registry.lookup/2","doc":"","ref":"Multiverses.Registry.html#lookup/2"},{"type":"function","title":"Multiverses.Registry.match/3","doc":"See Registry.match/3 .","ref":"Multiverses.Registry.html#match/3"},{"type":"function","title":"Multiverses.Registry.match/4","doc":"See Registry.match/4 .","ref":"Multiverses.Registry.html#match/4"},{"type":"function","title":"Multiverses.Registry.meta/2","doc":"See Registry.meta/2 .","ref":"Multiverses.Registry.html#meta/2"},{"type":"function","title":"Multiverses.Registry.put_meta/3","doc":"See Registry.put_meta/3 .","ref":"Multiverses.Registry.html#put_meta/3"},{"type":"function","title":"Multiverses.Registry.register/3","doc":"Registers the calling process with the Registry. Works as Registry.register/3 does.","ref":"Multiverses.Registry.html#register/3"},{"type":"function","title":"Multiverses.Registry.select/2","doc":"","ref":"Multiverses.Registry.html#select/2"},{"type":"function","title":"Multiverses.Registry.start_link/1","doc":"See Registry.start_link/1 .","ref":"Multiverses.Registry.html#start_link/1"},{"type":"function","title":"Multiverses.Registry.unregister/2","doc":"","ref":"Multiverses.Registry.html#unregister/2"},{"type":"function","title":"Multiverses.Registry.unregister_match/3","doc":"See Registry.unregister_match/3 .","ref":"Multiverses.Registry.html#unregister_match/3"},{"type":"function","title":"Multiverses.Registry.unregister_match/4","doc":"See Registry.unregister_match/4 .","ref":"Multiverses.Registry.html#unregister_match/4"},{"type":"function","title":"Multiverses.Registry.update_value/3","doc":"","ref":"Multiverses.Registry.html#update_value/3"},{"type":"function","title":"Multiverses.Registry.values/3","doc":"See Registry.values/3 .","ref":"Multiverses.Registry.html#values/3"},{"type":"function","title":"Multiverses.Registry.whereis_name/1","doc":"","ref":"Multiverses.Registry.html#whereis_name/1"},{"type":"module","title":"Multiverses.Server","doc":"core server for managing multiverse partitions","ref":"Multiverses.Server.html"},{"type":"function","title":"Multiverses.Server.all/1","doc":"","ref":"Multiverses.Server.html#all/1"},{"type":"function","title":"Multiverses.Server.allow/2","doc":"","ref":"Multiverses.Server.html#allow/2"},{"type":"function","title":"Multiverses.Server.allow/3","doc":"","ref":"Multiverses.Server.html#allow/3"},{"type":"function","title":"Multiverses.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Multiverses.Server.html#child_spec/1"},{"type":"function","title":"Multiverses.Server.clear/1","doc":"","ref":"Multiverses.Server.html#clear/1"},{"type":"function","title":"Multiverses.Server.id/2","doc":"","ref":"Multiverses.Server.html#id/2"},{"type":"function","title":"Multiverses.Server.shard/1","doc":"","ref":"Multiverses.Server.html#shard/1"},{"type":"function","title":"Multiverses.Server.shards/1","doc":"","ref":"Multiverses.Server.html#shards/1"},{"type":"module","title":"MultiversesTest.Replicant","doc":"allows you to replicate test module multiple times, forcing them to be run concurrently. This helps prove that the system can be run in parallel without issue. you should set the replication level with the REPLICATION environment variable, for example: REPLICATION=10 mix test","ref":"MultiversesTest.Replicant.html"},{"type":"macro","title":"MultiversesTest.Replicant.defmoduler/2","doc":"use in place of defmodule/2 for any test you'd like replicated.","ref":"MultiversesTest.Replicant.html#defmoduler/2"},{"type":"exception","title":"Multiverses.UnexpectedCallError","doc":"","ref":"Multiverses.UnexpectedCallError.html"},{"type":"extras","title":"Multiverses","doc":"","ref":"readme.html"},{"type":"extras","title":"Multiverses - Multiverses for Elixir.","doc":"Don't let Mox, Ecto, Hound and Wallaby have ALL the fun! Wubba dubba lub lub!","ref":"readme.html#multiverses-for-elixir"},{"type":"extras","title":"Multiverses - Usage","doc":"add a line into the configuration (typically test.exs ): If you'd like to drop in multiverse sharding for registry # config.exs config :my_app , Registry , Registry # test.exs config :my_app , Registry , Multiverses.Registry defmodule MyModule do @registry Application . config_env! ( :my_app , Registry ) def my_function ( ... ) do # uses Multiverses.Registry when enabled. @registry . unregister ( ... ) end end","ref":"readme.html#usage"},{"type":"extras","title":"Multiverses - Testing","doc":"you can activate multiple copies of all the tests by passing the REPLICATION system environment variable: REPLICATION=10 mix test will copy the test modules 10 times over, so multiple versions of the same module could possibly run simultaneously.","ref":"readme.html#testing"},{"type":"extras","title":"Multiverses - Installation","doc":"The package can be installed by adding multiverses to your list of dependencies in mix.exs : def deps do [ { :multiverses , &quot;~&gt; 0.11.0&quot; , only : :test } ] end Documentation can be found at https://hexdocs.pm/multiverses .","ref":"readme.html#installation"}]